@@ -159,7 +159,7 @@ impl<'a, R: Repr> Dumper<'a, R> {
 ///         "a" => "b",
 ///         "c" => "d",
 ///     }),
-/// ], &Default::default());
+/// ], &[]);
 /// let ans = "\
 /// a: b
 /// c: d
@@ -170,11 +170,17 @@ impl<'a, R: Repr> Dumper<'a, R> {
 /// When calling [`parse`] function then [`dump`] the string, the string can be reformatted.
 ///
 /// Anchors can pass with the result of the [`Loader`](crate::parser::Loader).
-pub fn dump<R: Repr>(nodes: &[Node<R>], anchors: &Anchors<R>) -> String {
+pub fn dump<R: Repr>(nodes: &[Node<R>], anchors: &[Anchors<R>]) -> String {
+    let anchors_empty = Anchors::new();
     nodes
         .iter()
         .enumerate()
         .map(|(i, node)| {
+            let anchors = if i < anchors.len() {
+                &anchors[i]
+            } else {
+                &anchors_empty
+            };
             let doc = Dumper::new(node, anchors).dump() + NL;
             match i {
                 0 => doc,
