@@ -1,4 +1,4 @@
-use crate::{repr::*, *};
+use crate::{parser::Anchors, repr::*, *};
 use alloc::string::{String, ToString};
 use core::{
     fmt::{Debug, Formatter},
@@ -6,37 +6,21 @@ use core::{
     iter::FromIterator,
     marker::PhantomData,
     ops::Index,
-    str::FromStr,
 };
 
 macro_rules! as_method {
-    {$(#[$meta:meta])* fn $id:ident = $ty:ident$(($op:ident))?
+    {$($(#[$meta:meta])* fn $id:ident = $ty:ident$(($op:ident))?
         $(| ($default:expr)?)?
-        $(| $ty2:ident)* -> $r:ty} => {
+        $(| $ty2:ident)* -> $r:ty)+} => {$(
         $(#[$meta])*
         pub fn $id(&self) -> Result<$r, u64> {
             match self.yaml() {
                 Yaml::$ty(v) $(| Yaml::$ty2(v))* => Ok(v$(.$op())?),
                 $(Yaml::Null => Ok($default),)?
-                _ => Err(self.pos()),
+                _ => Err(self.pos),
             }
         }
-    };
-}
-
-macro_rules! as_num_method {
-    {$(#[$meta:meta])* fn $id:ident = $ty1:ident $(| $ty2:ident)*} => {
-        $(#[$meta])*
-        pub fn $id<N: FromStr>(&self) -> Result<N, u64> {
-            match self.yaml() {
-                Yaml::$ty1(n) $(| Yaml::$ty2(n))* => match n.parse() {
-                    Ok(v) => Ok(v),
-                    Err(_) => Err(self.pos()),
-                },
-                _ => Err(self.pos()),
-            }
-        }
-    };
+    )+};
 }
 
 macro_rules! impl_iter {
@@ -129,18 +113,18 @@ pub type NodeArc = Node<ArcRepr>;
 pub struct Node<R: Repr> {
     pos: u64,
     tag: String,
-    yaml: R::Ty,
+    yaml: R::Rc,
     _marker: PhantomData<R>,
 }
 
 impl<R: Repr> Node<R> {
     /// Create node from YAML data.
     pub fn new(yaml: impl Into<Yaml<R>>, pos: u64, tag: impl ToString) -> Self {
-        Self::new_repr(R::repr(yaml.into()), pos, tag)
+        Self::new_repr(R::new_rc(yaml.into()), pos, tag)
     }
 
     /// Create from a representation.
-    pub fn new_repr(yaml: R::Ty, pos: u64, tag: impl ToString) -> Self {
+    pub fn new_repr(yaml: R::Rc, pos: u64, tag: impl ToString) -> Self {
         Self {
             yaml,
             pos,
@@ -151,11 +135,11 @@ impl<R: Repr> Node<R> {
 
     /// Set from existing YAML data.
     pub fn set_yaml(&mut self, yaml: impl Into<Yaml<R>>) {
-        self.set_repr(R::repr(yaml.into()));
+        self.set_repr(R::new_rc(yaml.into()));
     }
 
     /// Set from existing YAML representation.
-    pub fn set_repr(&mut self, yaml: R::Ty) {
+    pub fn set_repr(&mut self, yaml: R::Rc) {
         self.yaml = yaml;
     }
 
@@ -189,24 +173,28 @@ impl<R: Repr> Node<R> {
     }
 
     /// Clone YAML repr.
-    pub fn clone_yaml(&self) -> R::Ty {
+    pub fn clone_yaml(&self) -> R::Rc {
         self.yaml.clone()
     }
 
     /// As reference for the underlying reference counter.
     ///
     /// ```
-    /// use std::rc::Rc;
+    /// use std::{rc::Rc, sync::Arc};
     /// use yaml_peg::node;
     ///
-    /// let a = node!("a");
+    /// let a_rc = node!("a");
+    /// let a_arc = node!(arc "a");
     /// {
-    ///     let b = a.clone();
-    ///     assert_eq!(2, Rc::strong_count(b.rc_ref()));
+    ///     let b_rc = a_rc.clone();
+    ///     let b_arc = a_arc.clone();
+    ///     assert_eq!(2, Rc::strong_count(b_rc.rc_ref()));
+    ///     assert_eq!(2, Arc::strong_count(b_arc.rc_ref()));
     /// }
-    /// assert_eq!(1, Rc::strong_count(a.rc_ref()));
+    /// assert_eq!(1, Rc::strong_count(a_rc.rc_ref()));
+    /// assert_eq!(1, Arc::strong_count(a_arc.rc_ref()));
     /// ```
-    pub fn rc_ref(&self) -> &R::Ty {
+    pub fn rc_ref(&self) -> &R::Rc {
         &self.yaml
     }
 
@@ -215,6 +203,50 @@ impl<R: Repr> Node<R> {
         *self.yaml() == Yaml::Null
     }
 
+    /// Convert to integer.
+    ///
+    /// ```
+    /// use yaml_peg::node;
+    ///
+    /// assert_eq!(60, node!(60).as_int().unwrap());
+    /// ```
+    pub fn as_int(&self) -> Result<i64, u64> {
+        match self.yaml() {
+            Yaml::Int(s) => to_i64(s).map_err(|_| self.pos),
+            _ => Err(self.pos),
+        }
+    }
+
+    /// Convert to float.
+    ///
+    /// ```
+    /// use yaml_peg::node;
+    ///
+    /// assert_eq!(20.06, node!(20.06).as_float().unwrap());
+    /// ```
+    pub fn as_float(&self) -> Result<f64, u64> {
+        match self.yaml() {
+            Yaml::Float(s) => to_f64(s).map_err(|_| self.pos),
+            _ => Err(self.pos),
+        }
+    }
+
+    /// Convert to float for any number.
+    ///
+    /// ```
+    /// use yaml_peg::node;
+    ///
+    /// assert_eq!(60., node!(60).as_number().unwrap());
+    /// assert_eq!(20.06, node!(20.06).as_number().unwrap());
+    /// ```
+    pub fn as_number(&self) -> Result<f64, u64> {
+        match self.yaml() {
+            Yaml::Int(s) => to_i64(s).map(|n| n as f64).map_err(|_| self.pos),
+            Yaml::Float(s) => to_f64(s).map_err(|_| self.pos),
+            _ => Err(self.pos),
+        }
+    }
+
     as_method! {
         /// Convert to boolean.
         ///
@@ -224,43 +256,7 @@ impl<R: Repr> Node<R> {
         /// assert!(node!(true).as_bool().unwrap());
         /// ```
         fn as_bool = Bool(clone) -> bool
-    }
 
-    as_num_method! {
-        /// Convert to integer.
-        ///
-        /// ```
-        /// use yaml_peg::node;
-        ///
-        /// assert_eq!(60, node!(60).as_int().unwrap());
-        /// ```
-        fn as_int = Int
-    }
-
-    as_num_method! {
-        /// Convert to float.
-        ///
-        /// ```
-        /// use yaml_peg::node;
-        ///
-        /// assert_eq!(20.06, node!(20.06).as_float().unwrap());
-        /// ```
-        fn as_float = Float
-    }
-
-    as_num_method! {
-        /// Convert to number.
-        ///
-        /// ```
-        /// use yaml_peg::node;
-        ///
-        /// assert_eq!(60, node!(60).as_number().unwrap());
-        /// assert_eq!(20.06, node!(20.06).as_number().unwrap());
-        /// ```
-        fn as_number = Int | Float
-    }
-
-    as_method! {
         /// Convert to string pointer.
         ///
         /// This method allows null, it represented as empty string.
@@ -273,6 +269,33 @@ impl<R: Repr> Node<R> {
         /// assert!(node!(()).as_str().unwrap().is_empty());
         /// ```
         fn as_str = Str | ("")? -> &str
+
+        /// Convert to sequence.
+        ///
+        /// ```
+        /// use yaml_peg::node;
+        ///
+        /// let n = node!(["55"]);
+        /// assert_eq!(node!("55"), n.as_seq().unwrap()[0]);
+        /// for n in n.as_seq().unwrap() {
+        ///     assert_eq!(node!("55"), n);
+        /// }
+        /// ```
+        fn as_seq = Seq(clone) -> Seq<R>
+
+        /// Convert to map.
+        ///
+        /// ```
+        /// use yaml_peg::node;
+        ///
+        /// let n = node!({1 => 2});
+        /// assert_eq!(node!(2), n.as_map().unwrap()[&node!(1)]);
+        /// for (k, v) in n.as_map().unwrap() {
+        ///     assert_eq!(node!(1), k);
+        ///     assert_eq!(node!(2), v);
+        /// }
+        /// ```
+        fn as_map = Map(clone) -> Map<R>
     }
 
     /// Convert to string pointer for string, null, bool, int, and float type.
@@ -295,39 +318,28 @@ impl<R: Repr> Node<R> {
             Yaml::Bool(true) => Ok("true"),
             Yaml::Bool(false) => Ok("false"),
             Yaml::Null => Ok(""),
-            _ => Err(self.pos()),
+            _ => Err(self.pos),
         }
     }
 
-    as_method! {
-        /// Convert to sequence.
-        ///
-        /// ```
-        /// use yaml_peg::node;
-        ///
-        /// let n = node!(["55"]);
-        /// assert_eq!(node!("55"), n.as_seq().unwrap()[0]);
-        /// for n in n.as_seq().unwrap() {
-        ///     assert_eq!(node!("55"), n);
-        /// }
-        /// ```
-        fn as_seq = Seq(clone) -> Seq<R>
-    }
-
-    as_method! {
-        /// Convert to map.
-        ///
-        /// ```
-        /// use yaml_peg::node;
-        ///
-        /// let n = node!({1 => 2});
-        /// assert_eq!(node!(2), n.as_map().unwrap()[&node!(1)]);
-        /// for (k, v) in n.as_map().unwrap() {
-        ///     assert_eq!(node!(1), k);
-        ///     assert_eq!(node!(2), v);
-        /// }
-        /// ```
-        fn as_map = Map(clone) -> Map<R>
+    /// Return the reference from `anchors` or self.
+    ///
+    /// ```
+    /// use yaml_peg::{node, parser::Anchors};
+    ///
+    /// let mut anchors = Anchors::new();
+    /// anchors.insert("a".to_string(), node!(20));
+    /// assert_eq!(
+    ///     20,
+    ///     node!(*"a").as_anchor(&anchors).unwrap().as_int().unwrap()
+    /// );
+    /// ```
+    pub fn as_anchor<'a>(&'a self, anchors: &'a Anchors<R>) -> Result<&'a Self, u64> {
+        if let Yaml::Alias(a) = self.yaml() {
+            anchors.get(a).ok_or(self.pos)
+        } else {
+            Ok(self)
+        }
     }
 
     /// Convert to map and try to get the value by key.
@@ -344,13 +356,9 @@ impl<R: Repr> Node<R> {
     /// ```
     pub fn get<Y: Into<Self>>(&self, key: Y) -> Result<&Self, u64> {
         if let Yaml::Map(m) = self.yaml() {
-            if let Some(n) = m.get(&key.into()) {
-                Ok(n)
-            } else {
-                Err(self.pos())
-            }
+            m.get(&key.into()).ok_or(self.pos)
         } else {
-            Err(self.pos())
+            Err(self.pos)
         }
     }
 
@@ -409,7 +417,7 @@ impl<R: Repr> Node<R> {
                 Ok(default)
             }
         } else {
-            Err(self.pos())
+            Err(self.pos)
         }
     }
 
@@ -425,34 +433,25 @@ impl<R: Repr> Node<R> {
     /// ```
     pub fn get_ind(&self, ind: Ind) -> Result<&Self, u64> {
         if let Yaml::Seq(a) = self.yaml() {
-            if let Some(n) = a.get(ind.0) {
-                Ok(n)
-            } else {
-                Err(self.pos())
-            }
+            a.get(ind.0).ok_or(self.pos)
         } else {
-            Err(self.pos())
+            Err(self.pos)
         }
     }
 }
 
 impl<R: Repr> Debug for Node<R> {
     fn fmt(&self, f: &mut Formatter<'_>) -> core::fmt::Result {
-        f.debug_struct("Node")
-            .field("pos", &self.pos)
-            .field("tag", &self.tag)
-            .field("yaml", &self.yaml)
-            .finish()
+        write!(f, "Node{:?}", &self.yaml)
     }
 }
 
 impl<R: Repr> Clone for Node<R> {
     fn clone(&self) -> Self {
         Self {
-            pos: self.pos,
             tag: self.tag.clone(),
             yaml: self.clone_yaml(),
-            _marker: PhantomData,
+            ..*self
         }
     }
 }
