@@ -1,5 +1,5 @@
 use super::*;
-use alloc::{format, string::ToString};
+use alloc::format;
 use core::cmp::Ordering;
 
 /// The low level grammar implementation for YAML.
@@ -27,11 +27,28 @@ impl Parser<'_> {
     /// Match integer.
     pub fn int(&mut self) -> PResult<String> {
         self.num_prefix()?;
-        let s = self.text();
+        let mut s = self.text();
+        if s.as_bytes() == b"0" && self.context(|p| p.octal().is_ok() || p.hexadecimal().is_ok()) {
+            s = self.text();
+        }
         self.bound()?;
         Ok(s)
     }
 
+    fn octal(&mut self) -> PResult<()> {
+        self.sym(b'o')?;
+        self.take_while(Self::ascii_digit(8), TakeOpt::More(1))
+    }
+
+    fn hexadecimal(&mut self) -> PResult<()> {
+        self.sym(b'x')?;
+        self.take_while(Self::ascii_digit(16), TakeOpt::More(1))
+    }
+
+    fn ascii_digit(i: u8) -> impl Fn(&u8) -> bool + 'static {
+        move |c| c.is_ascii_digit() || (*c > b'a' && *c < b'a' + i) || (*c > b'A' && *c < b'A' + i)
+    }
+
     /// Match float.
     pub fn float(&mut self) -> PResult<String> {
         self.num_prefix()?;
@@ -172,7 +189,7 @@ impl Parser<'_> {
         Ok(chomp(s))
     }
 
-    /// Match chomping option.
+    /// Match string chomping option.
     pub fn chomp(&mut self) -> impl Fn(String) -> String {
         self.context(|p| {
             if p.sym(b'-').is_ok() {
@@ -269,7 +286,7 @@ impl Parser<'_> {
                     // Tag prefix variable
                     p.tag[&tag].clone()
                 } else {
-                    tag
+                    String::new()
                 }
             } else if p.sym(b'<').is_ok() {
                 // Full tag
