@@ -49,12 +49,12 @@ pub use self::{
 };
 use crate::{repr::Repr, *};
 use alloc::{
+    collections::BTreeMap,
     string::{String, ToString},
     vec,
     vec::Vec,
 };
 use core::ops::{Deref, DerefMut};
-use ritelinked::LinkedHashMap;
 
 mod base;
 mod error;
@@ -81,7 +81,7 @@ macro_rules! or {
 pub(crate) use tag_prefix;
 
 /// The type of anchor recorder in the [`Loader`].
-pub type Anchors<R> = LinkedHashMap<String, Node<R>>;
+pub type Anchors<R> = BTreeMap<String, Node<R>>;
 /// The default prefix of the YAML sub tag.
 pub const DEFAULT_PREFIX: &str = tag_prefix!();
 
@@ -114,8 +114,9 @@ pub const DEFAULT_PREFIX: &str = tag_prefix!();
 pub struct Loader<'a, R: Repr> {
     /// Parser base.
     pub parser: Parser<'a>,
-    keep_anchors: bool,
-    anchors: Anchors<R>,
+    cyclic_mode: bool,
+    anchors: Vec<Anchors<R>>,
+    doc_ind: usize,
 }
 
 impl<'a, R: Repr> Loader<'a, R> {
@@ -123,8 +124,9 @@ impl<'a, R: Repr> Loader<'a, R> {
     pub fn new(doc: &'a [u8]) -> Self {
         Self {
             parser: Parser::new(doc),
-            keep_anchors: false,
-            anchors: Anchors::new(),
+            cyclic_mode: false,
+            anchors: Vec::new(),
+            doc_ind: 0,
         }
     }
 }
@@ -147,17 +149,20 @@ impl<'a, R: Repr> Loader<'a, R> {
 impl<R: Repr> Loader<'_, R> {
     /// Keep the anchor insertion.
     ///
+    /// + Allow alias used before undefined anchor created. E.g., cyclic data.
+    /// + Keep anchors for [`crate::dump`] function.
+    ///
     /// This will make [`Yaml::Alias`] have a placeholder
     /// and adding anchor information in the [`Node`].
-    pub fn keep_anchors(self, keep_anchors: bool) -> Self {
+    pub fn cyclic_mode(self, cyclic_mode: bool) -> Self {
         Self {
-            keep_anchors,
+            cyclic_mode,
             ..self
         }
     }
 
     /// Consume this loader and return the recorded anchors.
-    pub fn get_anchors(self) -> Anchors<R> {
+    pub fn get_anchors(self) -> Vec<Anchors<R>> {
         self.anchors
     }
 
@@ -172,8 +177,6 @@ impl<R: Repr> Loader<'_, R> {
         }
         self.gap(true).unwrap_or_default();
         self.sym_seq(b"---").unwrap_or_default();
-        self.gap(true).unwrap_or_default();
-        self.forward();
         let mut v = vec![self.doc()?];
         loop {
             self.gap(true).unwrap_or_default();
@@ -183,8 +186,6 @@ impl<R: Repr> Loader<'_, R> {
             if self.sym_seq(b"---").is_err() {
                 return self.err("document splitter");
             }
-            self.gap(true).unwrap_or_default();
-            self.forward();
             v.push(self.doc()?);
         }
         Ok(v)
@@ -192,12 +193,14 @@ impl<R: Repr> Loader<'_, R> {
 
     /// Match one doc block.
     pub fn doc(&mut self) -> PResult<Node<R>> {
-        self.ind_define(0)?;
+        self.context(|p| p.bound().unwrap_or_default());
         self.forward();
+        self.anchors.push(Anchors::new());
         let ret = self.scalar(0, false, false)?;
         self.gap(true).unwrap_or_default();
         self.sym_seq(b"...").unwrap_or_default();
         self.forward();
+        self.doc_ind += 1;
         Ok(ret)
     }
 
@@ -220,9 +223,9 @@ impl<R: Repr> Loader<'_, R> {
     pub fn scalar(&mut self, level: usize, nest: bool, inner: bool) -> PResult<Node<R>> {
         self.scalar_node(|p| {
             if let Ok(s) = p.string_literal(level) {
-                Ok(R::repr(Yaml::Str(s)))
+                Ok(R::new_rc(Yaml::Str(s)))
             } else if let Ok(s) = p.string_folded(level) {
-                Ok(R::repr(Yaml::Str(s)))
+                Ok(R::new_rc(Yaml::Str(s)))
             } else {
                 or!(
                     p.seq(level, nest);
@@ -240,9 +243,9 @@ impl<R: Repr> Loader<'_, R> {
 
     fn scalar_node<F>(&mut self, f: F) -> PResult<Node<R>>
     where
-        F: FnOnce(&mut Self) -> PResult<R::Ty>,
+        F: FnOnce(&mut Self) -> PResult<R::Rc>,
     {
-        let anchor = self.anchor().unwrap_or_default();
+        let mut anchor = self.anchor().unwrap_or_default();
         if !anchor.is_empty() {
             self.bound()?;
         }
@@ -252,11 +255,25 @@ impl<R: Repr> Loader<'_, R> {
             self.bound()?;
         }
         self.forward();
+        let anchor2 = self.anchor().unwrap_or_default();
+        if !anchor2.is_empty() {
+            if anchor.is_empty() {
+                self.bound()?;
+                anchor = anchor2;
+            } else {
+                return self.err("duplicated anchor definition");
+            }
+        }
+        self.forward();
         let pos = self.indicator();
         let yaml = f(self)?;
         self.forward();
         let node = Node::new_repr(yaml, pos, &tag);
-        if !anchor.is_empty() && self.anchors.insert(anchor, node.clone()).is_some() {
+        if !anchor.is_empty()
+            && self.anchors[self.doc_ind]
+                .insert(anchor, node.clone())
+                .is_some()
+        {
             self.err("duplicated anchor definition")
         } else {
             Ok(node)
@@ -264,29 +281,27 @@ impl<R: Repr> Loader<'_, R> {
     }
 
     /// Match flow scalar terminal.
-    pub fn scalar_term(&mut self, level: usize, inner: bool) -> PResult<R::Ty> {
+    pub fn scalar_term(&mut self, level: usize, inner: bool) -> PResult<R::Rc> {
         let yaml = if let Ok(s) = self.float() {
-            R::repr(Yaml::Float(s))
+            R::new_rc(Yaml::Float(s))
         } else if let Ok(s) = self.sci_float() {
-            R::repr(Yaml::Float(s))
+            R::new_rc(Yaml::Float(s))
         } else if let Ok(s) = self.int() {
-            R::repr(Yaml::Int(s))
+            R::new_rc(Yaml::Int(s))
         } else if let Ok(s) = self.anchor_use() {
-            if let Some(node) = self.anchors.get(&s) {
-                if self.keep_anchors {
-                    R::repr(Yaml::Alias(s))
-                } else {
-                    node.clone_yaml()
-                }
+            if self.cyclic_mode {
+                R::new_rc(Yaml::Alias(s))
+            } else if let Some(node) = self.anchors[self.doc_ind].get(&s) {
+                node.clone_yaml()
             } else {
                 return self.err("anchor referenced before definition");
             }
         } else if let Ok(s) = self.string_quoted(b'\'', b"''") {
-            R::repr(Yaml::Str(s))
+            R::new_rc(Yaml::Str(s))
         } else if let Ok(s) = self.string_quoted(b'"', b"\\\"") {
-            R::repr(Yaml::Str(Parser::escape(&s)))
+            R::new_rc(Yaml::Str(Parser::escape(&s)))
         } else if let Ok(s) = self.string_plain(level, inner) {
-            R::repr(match s.as_str() {
+            R::new_rc(match s.as_str() {
                 "~" | "null" | "Null" | "NULL" => Yaml::Null,
                 "true" | "True" | "TRUE" => Yaml::Bool(true),
                 "false" | "False" | "FALSE" => Yaml::Bool(false),
@@ -299,14 +314,14 @@ impl<R: Repr> Loader<'_, R> {
             or!(
                 self.seq_flow(level);
                 self.map_flow(level);
-                Ok(R::repr(Yaml::Null))
+                Ok(R::new_rc(Yaml::Null))
             )?
         };
         Ok(yaml)
     }
 
     /// Match flow sequence.
-    pub fn seq_flow(&mut self, level: usize) -> PResult<R::Ty> {
+    pub fn seq_flow(&mut self, level: usize) -> PResult<R::Rc> {
         self.sym(b'[')?;
         let mut v = vec![];
         loop {
@@ -328,11 +343,11 @@ impl<R: Repr> Loader<'_, R> {
             }
         }
         self.forward();
-        Ok(R::repr(v.into_iter().collect()))
+        Ok(R::new_rc(v.into_iter().collect()))
     }
 
     /// Match flow map.
-    pub fn map_flow(&mut self, level: usize) -> PResult<R::Ty> {
+    pub fn map_flow(&mut self, level: usize) -> PResult<R::Rc> {
         self.sym(b'{')?;
         let mut m = vec![];
         loop {
@@ -355,9 +370,10 @@ impl<R: Repr> Loader<'_, R> {
             } else {
                 or!(self.scalar_flow(level + 1, true); self.err("flow map key"))?
             };
-            if self.sym(b':').is_err() || self.bound().is_err() {
+            if self.sym(b':').is_err() {
                 return self.err("flow map splitter");
             }
+            self.context(|p| p.bound().unwrap_or_default());
             self.forward();
             let v = or!(
                 self.scalar(level + 1, false, true);
@@ -371,11 +387,11 @@ impl<R: Repr> Loader<'_, R> {
             }
         }
         self.forward();
-        Ok(R::repr(m.into_iter().collect()))
+        Ok(R::new_rc(m.into_iter().collect()))
     }
 
     /// Match sequence.
-    pub fn seq(&mut self, level: usize, nest: bool) -> PResult<R::Ty> {
+    pub fn seq(&mut self, level: usize, nest: bool) -> PResult<R::Rc> {
         let mut v = vec![];
         loop {
             self.forward();
@@ -415,11 +431,11 @@ impl<R: Repr> Loader<'_, R> {
         }
         // Keep last wrapping
         self.backward();
-        Ok(R::repr(v.into_iter().collect()))
+        Ok(R::new_rc(v.into_iter().collect()))
     }
 
     /// Match map.
-    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> PResult<R::Ty> {
+    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> PResult<R::Rc> {
         let mut m = vec![];
         loop {
             self.forward();
@@ -477,7 +493,7 @@ impl<R: Repr> Loader<'_, R> {
         }
         // Keep last wrapping
         self.backward();
-        Ok(R::repr(m.into_iter().collect()))
+        Ok(R::new_rc(m.into_iter().collect()))
     }
 }
 
@@ -495,7 +511,7 @@ impl<R: Repr> DerefMut for Loader<'_, R> {
     }
 }
 
-/// Parse YAML document into [`alloc::rc::Rc`] or [`alloc::sync::Arc`] data holder.
+/// Parse non-cyclic YAML document into [`alloc::rc::Rc`] or [`alloc::sync::Arc`] data holder.
 /// Return an sequence of nodes and insert the anchors automatically.
 ///
 /// ```
@@ -522,8 +538,25 @@ impl<R: Repr> DerefMut for Loader<'_, R> {
 ///     "age" => 46,
 /// })]);
 /// ```
-///
-/// Please see [`Loader`] for parser customization.
 pub fn parse<R: Repr>(doc: &str) -> Result<Seq<R>, PError> {
     Loader::new(doc.as_bytes()).parse()
 }
+
+/// Parse cyclic YAML document into [`alloc::rc::Rc`] or [`alloc::sync::Arc`] data holder.
+/// Return an sequence of nodes and keep the anchors placeholder.
+///
+/// ```
+/// use yaml_peg::{parse_cyclic, node};
+///
+/// let doc = "
+/// --- &root
+/// map: *root
+/// ";
+/// let (root, anchors) = parse_cyclic(doc).unwrap();
+/// assert_eq!(vec![node!({"map" => node!(*"root")})], root);
+/// assert_eq!(anchors[0].get("root").unwrap(), &root[0]);
+/// ```
+pub fn parse_cyclic<R: Repr>(doc: &str) -> Result<(Seq<R>, Vec<Anchors<R>>), PError> {
+    let mut loader = Loader::new(doc.as_bytes()).cyclic_mode(true);
+    loader.parse().map(|root| (root, loader.get_anchors()))
+}
